﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Microsoft.SqlServer.Management.Smo;
using System.Data.SqlClient;
using Microsoft.SqlServer.Management.Common;

#region History
/*
 * Created and Designed By: Osama
 */
#endregion

namespace CreateDynamicTables
{
    public partial class frmCreateTable : Form
    {
        public frmCreateTable()
        {
            InitializeComponent();
        }

        public static string APPLICATION_NAME = "Create Dynamic Table";
        public static string DATABASE_NAME = "Student"; // put the name of database in which you want to create table
        
        // and change the CONNECTION_STRING according tou your sql server
        public static string CONNECTION_STRING = @"Data Source=.;Initial Catalog=" + DATABASE_NAME + ";Integrated Security=true";

        public static void ShowMessage(string message, MessageBoxIcon icon)
        {
            MessageBox.Show(message, APPLICATION_NAME, MessageBoxButtons.OK, icon);
        }

        /// <summary>
        /// Filling DataGridView with fieldNames, file is available in Applcation Exe Path (fieldNames.txt)
        /// </summary>
        public void FillDataGridView()
        {
            TextReader tr = File.OpenText("fieldNames.txt");
            string fieldName = tr.ReadLine();

            while (fieldName != null)
            {
                int row = dgvFieldNames.Rows.Add();

                dgvFieldNames.Rows[row].Cells["colFieldName"].Value = fieldName;
                dgvFieldNames.Rows[row].Cells["colInclude"].Value = false;

                fieldName = tr.ReadLine();
            }                
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            try
            {                
                FillDataGridView();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);                
            }
        }

        private void btnDone_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        public void Create(string tableName)
        {
            if (String.IsNullOrEmpty(tableName))
            {
                ShowMessage("برای ایجاد در پایگاه داده برای جدول نام تعریف کنید.", MessageBoxIcon.Information);
                return;
            }

            if (!IsFieldIncluded())
            {
                ShowMessage("You must select field(s) from data grid to create columns of new table.", MessageBoxIcon.Information);
                return;
            }

            List<string> selectedFields = GetSelectedFieldNames();

            // at least one column other than Primary key ("ID") that is necessary in each table
            // to uniquely define table data.
            if (selectedFields.Count > 0)
            {
                CreateTable(tableName, selectedFields);
            }     
        }

        /// <summary>
        /// It gets table name and list of column names that are included in that table.
        /// </summary>
        /// <param name="tableName"></param>
        /// <param name="columnNames"></param>
        public void CreateTable(string tableName, List<string> columnNames)
        {
            SqlConnection sqlCon = new SqlConnection(CONNECTION_STRING);
            Server server = new Server(new ServerConnection(sqlCon));
                     
            Database dbName = server.Databases[DATABASE_NAME];

            Table newTable = new Table(dbName, tableName);           

            // creating Primary key column ("ID")
            Column idColumn = new Column(newTable, "ID");
            idColumn.DataType = DataType.Int;
            idColumn.Nullable = false;
            idColumn.Identity = true;
            idColumn.IdentitySeed = 1;
            idColumn.IdentityIncrement = 1;

            newTable.Columns.Add(idColumn);

            if (columnNames.Count > 0)
            {
                foreach (var colName in columnNames)
                {                                        
                    if (colName.ToUpper().Equals("Gender".ToUpper()) || colName.ToUpper().Equals("Sex".ToUpper()))
                    {
                        Column col = new Column(newTable, colName);
                        col.DataType = DataType.Bit;
                        col.Nullable = true;
                        newTable.Columns.Add(col);
                    }
                    else if (colName.ToUpper().Equals("Date Of Birth".ToUpper()))
                    {
                        Column col = new Column(newTable, colName);
                        col.DataType = DataType.DateTime;
                        col.Nullable = true;
                        newTable.Columns.Add(col);
                    }
                    else
                    {
                        Column col = new Column(newTable, colName);
                        col.DataType = DataType.VarChar(50);
                        col.Nullable = true;
                        newTable.Columns.Add(col);
                    }
                }

                newTable.Create();                
            }

            txtTableName.Clear();
            ShowMessage("Table created successfully.", MessageBoxIcon.Information);                        
        }

        private void btnCreate_Click(object sender, EventArgs e)
        {
            try
            {
                Create(txtTableName.Text);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        /// <summary>
        /// Check if any row of data grid Include column is selected or not.
        /// </summary>
        /// <returns>true if Included single column else false</returns>
        public bool IsFieldIncluded()
        {
            bool isIncluded = false;

            foreach (DataGridViewRow dgvRow in dgvFieldNames.Rows)
            {
                if (Convert.ToBoolean(dgvRow.Cells["colInclude"].Value.ToString()))
                {
                    isIncluded = true;
                    return isIncluded;
                }
            }

            return isIncluded;
        }

        /// <summary>
        /// Gets the list of selected fields from data grid that would be the names of column
        /// in newly created Table.
        /// </summary>
        /// <returns></returns>
        public List<string> GetSelectedFieldNames()
        {
            List<string> fields = new List<string>();
            bool isIncluded = false;

            foreach (DataGridViewRow dgvRow in dgvFieldNames.Rows)
            {
                isIncluded = Convert.ToBoolean(dgvRow.Cells["colInclude"].Value.ToString());

                if (isIncluded)
                    fields.Add(dgvRow.Cells["colFieldName"].Value.ToString());
            }

            return fields;
        }

        private void btnDone_Click_1(object sender, EventArgs e)
        {

        }
    }
}
